#include <algorithm>
#include <cmath>
#include "mbed.h"
#include "rtos.h"
#include "SeeedLedBar.h"
#include "SeeedFourDigitDisp.h"
#include "SeeedQTouch.h"
#include "SeeedChainableLED.h"

Serial pc(SERIAL_TX, SERIAL_RX);
DigitalOut led(PF_12);
AnalogIn knob(PA_3);

void led_thread(void const* args) {
    while(1) {
        led = !led;
        Thread::wait(1000);
    }
}

int main() {
    pc.printf("\n\nstarting algorithm\n\n");
    SeeedLedBar ledBar = SeeedLedBar(PE_9, PF_13);
    SeeedFourDigitDisp disp = SeeedFourDigitDisp(PE_13, PF_15);
    SeeedQTouch qTouch = SeeedQTouch(PB_9, PB_8);
    SeeedChainableLED led_chain = SeeedChainableLED(PE_11, PF_14);

    Thread blinky(led_thread);

    disp.set_digit(0,0);
    int knob_led_bar[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    ledBar.ten_on();
    disp.clear_display();
    led_chain.turn_on();
    led_chain.set_color_rgb(100,200,100);
    while (1) {
        float led_frac = knob.read();
        int led_percent = floor(led_frac * 100);
        int tens = floor(led_frac * 10);
        int ones = led_percent % 10;
        ones = floor(ones * 10 / 8);
        if (ones > 8) ones = 8;
        for (int i = 0; i < tens; ++i) knob_led_bar[i] = 0xff;
        knob_led_bar[tens] = ones * 0xf;
        for (int i = ++tens; i < 10; ++i) knob_led_bar[i] = 0x00;
        ledBar.ten_set(knob_led_bar);
        disp.set_integer(led_percent);
        int eightBitInput = led_frac * 255;
        led_chain.set_color_rgb(eightBitInput, eightBitInput, eightBitInput);
        Thread::wait(10);
    }
}
